----------------------------------------------------------------------------------
-- Project Name: Gage Hash Function, C: 224 and r: 8
-- 
-- Copyright 2019:
--     Mohamed El-Hadedy <mealy@cpp.edu>
--     Danilo Gligoroski <danilog@ntnu.no>
-- 
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation; either version 2 of the
-- License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-- 02110-1301, USA.
-- 
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- Company: California State Polytechnic University, Pomona, Norwegian University of Science and Technology (NTNU)
-- Engineer-1: Assistant Professor at ECE-department, College of Engineering 
-- Engineer-2: Professor at Department of Information Security and Communication Technology
-- 
-- Create Date: 06/22/2019 02:06:31 PM
-- Design Name-1: Mohamed El-Hadedy, also Airforce Research Lab(AFRL) fellow, US-AFA
-- Design Name-2: Danilo Gligoroski, also Visiting professor: Department of Mathematics and Statistics, University of South Florida, USA

-- Module Name: Buffer - Behavioral
-- Project Name: Gage Hash Function , C: 224 and r: 8
-- Target Devices: xc7a35tcpg236-1 
-- Tool Versions: Vivado 2019.1
-- Description: Hash function Engine (FIFO for storing the message coming from the main testbench (can be coming from any serial communication protocol as PCI-E or UART, USB, SPI, I2C or more)
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use ieee.STD_LOGIC_ARITH.all;
use ieee.std_logic_unsigned.all;
use IEEE.math_real.all;
-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity OST_RAM is
    generic (c:positive:=224 ; r:positive:= 8);
    Port (
            clk   : in  std_logic;
            RAM_wr: in  std_logic;  
            PTR_wr: in  std_logic_vector(integer(ceil(log2(real(c+r))))-1 downto 0); 
        --    PTR_wr_b1: in  std_logic_vector(integer(ceil(log2(real(c+r))))-1 downto 0); 

            ST_INP: in  std_logic; 
      --      ST_INP_b1: in  std_logic; 

            RAM_rd: in  std_logic; 
            PTR_rd: in  std_logic_vector(integer(ceil(log2(real(c+r))))-1 downto 0); 
            ST_OP : out std_logic 
          );
end OST_RAM;

architecture Behavioral of OST_RAM is

type STATE_ARRAY is array (0 to 2**(integer(ceil(log2(real(c+r)))))-1) of std_logic;
signal RAM: STATE_ARRAY; 
begin
WRITE_PROCESS:process(clk)
        begin
        if(rising_edge(clk)) then
             if(RAM_wr='1') then -- when write enable = 1, 
                 RAM(conv_integer(unsigned(PTR_wr))) <= ST_INP;
           --      RAM(conv_integer(unsigned(PTR_wr_b1))) <= ST_INP_b1;
             end if;
         end if;
end process;

READ_PROCESS:process(clk)
        begin
         if(rising_edge(clk)) then
             if(RAM_rd='1') then -- when write enable = 1, 
                 ST_OP <= RAM(conv_integer(unsigned(PTR_rd)));
             end if;
         end if;
end process;

 -- Data to be read out 

end Behavioral;
