----------------------------------------------------------------------------------
-- Project Name: Gage Hash Function, C: 224 and r: 8
-- 
-- Copyright 2019:
--     Mohamed El-Hadedy <mealy@cpp.edu>
--     Danilo Gligoroski <danilog@ntnu.no>
-- 
-- This program is free software; you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation; either version 2 of the
-- License, or (at your option) any later version.
-- 
-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-- General Public License for more details.
-- 
-- You should have received a copy of the GNU General Public License
-- along with this program; if not, write to the Free Software
-- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-- 02110-1301, USA.
-- 
----------------------------------------------------------------------------------
----------------------------------------------------------------------------------
-- Company: California State Polytechnic University In Pomona 
-- Engineer: Assistant Professor at ECE-department, College of Engineering 
-- 
-- Create Date: 07/08/2019 01:56:37 PM
-- Design Name: Mohamed El-Hadedy 
-- Module Name: Q_sbox - Behavioral
-- Project Name: Gage and InGage 
-- Target Devices: Basys3 
-- Tool Versions: Vivado 2019.1
-- Description: SBOX takes 4-bit input and translate to 2-bit equivalents 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use ieee.STD_LOGIC_ARITH.all;
use ieee.std_logic_unsigned.all;
use IEEE.math_real.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Q_sbox is
  Port   (
         clk    : in  std_logic;
         rst    : in  std_logic; 
         EN_IN  : in  std_logic; 
         INP    : in  std_logic_vector(3 downto 0);
         OP     : out std_logic;
         EN_OUT : out std_logic;
         RAM_OUT: out std_logic     
         );
end Q_sbox;

architecture Behavioral of Q_sbox is

signal OP_tmp: std_logic_vector(1 downto 0); 
signal EN_ROM: std_logic;
signal EN_D0 : std_logic;
signal EN_D1 : std_logic; 
signal sel         : std_logic;

begin



-- here we need a LUT to setup the functionality of the Q-sbox 

SBOX: process(clk,rst) begin
    if (rst = '1') then 
        OP_tmp <= ("00");
        EN_ROM <= '0';
    elsif (rising_edge(clk)) then 
    if (EN_IN = '1') then 
    cont:case (INP) is 
        when "0000" => OP_tmp <= "01";
        when "0001" => OP_tmp <= "00";
        when "0010" => OP_tmp <= "11";
        when "0011" => OP_tmp <= "10";
        when "0100" => OP_tmp <= "00";
        when "0101" => OP_tmp <= "10";
        when "0110" => OP_tmp <= "01";
        when "0111" => OP_tmp <= "11";
        when "1000" => OP_tmp <= "10";
        when "1001" => OP_tmp <= "11";
        when "1010" => OP_tmp <= "00";
        when "1011" => OP_tmp <= "01";
        when "1100" => OP_tmp <= "11";
        when "1101" => OP_tmp <= "01";
        when "1110" => OP_tmp <= "10";
        when "1111" => OP_tmp <= "00";
        when others => OP_tmp <= (others =>'Z');
    end case cont; 
    end if; 
        EN_ROM <= EN_IN;
   end if; 
end process SBOX; 


OP <= ((not sel and OP_tmp(1) ) or (sel and OP_tmp(0))); 


D0:process(clk,rst)
   begin 
    if (rst = '1') then 
        EN_D0 <= '0';
    elsif(rising_edge(clk)) then 
        EN_D0 <= EN_ROM;
    end if; 
   end process; 
   
--D1:process(clk,rst)
--   begin 
--    if (rst = '1') then 
--       EN_D1 <= '0';
--    elsif(rising_edge(clk)) then 
--       EN_D1 <= EN_D0;
--    end if;    
--   end process; 
   
   sel <= not EN_ROM and EN_D0;
--   EN_OUT <= EN_IN or EN_ROM; 
    RAM_OUT<= EN_IN or EN_ROM;
    EN_OUT <= EN_ROM or EN_D0; 
end Behavioral;
